/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "file_stream.hh"
#include "lsmp_prod.hh"
#include "SigFlag.hh"
#include <string>
#include <fstream>
#include <iostream>

using namespace std;

int    debug = 0;
double delay = 0;

void
syntax(void) {
   cout << "Command line syntax: " << endl;
   cout << "smpull [-v] [--delay <t>] <shm-dir> <partition>" << endl;
}

int
main(int argc, const char* argv[]) {
   if (argc <=2) {
      syntax();
      return 1;
   }
   for (int i=1; i < argc-2; i++) {
      string argi = argv[i];
      if (argi == "-v") {
	 debug += 1;
      }
      else if (argi == "--delay") {
	 delay = strtod(argv[++i], 0);
      }
      else {
	 cerr << "smpull: unrecognized argument: " << argi << endl;
      }
   }

   file_stream fs(argv[argc - 2]);
   fs.set_debug(debug);

   LSMP_PROD part(argv[argc -1]);
   streampos parlen =part.getBufferLength();

   SigFlag term(SIGTERM);
   while (!term) {
      if (!fs.wait_file()) continue;
      while (fs.test_file()) {
         string file = fs.next_file();
	 if (debug) cout << "read next file: " << file << endl;

	 //----------------------------  Delay if necessary
	 if (delay > 0) {
	    struct timespec tm;
	    tm.tv_sec  = time_t(delay);
	    tm.tv_nsec = long((delay - double(tm.tv_sec)) * 1e9 + 0.5);
	    if (nanosleep(&tm, 0) < 0) {
	       perror("Error during delay");
	    }
	 }

	 //----------------------------  Parse the gps time
	 string gps_string = file;
	 size_t dirindex = gps_string.find("/");
	 while (dirindex != string::npos) {
	    gps_string.erase(0, dirindex+1);
	    dirindex = gps_string.find("/");
	 }
	 for (size_t i=0; i<2; i++) {
	    size_t inx = gps_string.find("-");
	    if (inx == string::npos) break;
	    if (inx == 10 && gps_string.find_first_not_of("0123456789") == inx) 
	        break;
	    gps_string.erase(0, inx+1);
	 }
	 size_t inx = 0;
	 uint32_t gps = stoul(gps_string, &inx);
	 if (gps_string[inx] != '-') {
	    static bool printed = false;
	    if (!printed) cerr << "file name is not standard meta-data format"
			       << endl;
	    printed = true;
	 }
	 
	 //----------------------------- Open the file
         ifstream is(file.c_str(), std::ifstream::binary);
         if (!is.is_open()) {
	    cerr << "Unable to open file: " << file << endl;
	    continue;
         }
         //----------------------------  Get length of file
         is.seekg (0, is.end);
         streampos length = is.tellg();
	 if (debug) cout << "file length: " << length << endl;
         is.seekg (0, is.beg);
         if (length > parlen) continue;

         //----------------------------  Get a buffer address
         char* buf = part.get_buffer();
         if (!buf) {
	    cerr << "Unable to get a buffer from partition: " << argv[2] << endl;           continue;
         }

         //----------------------------  read the frame
         is.read(buf, length);
         int readlen = is.gcount();
	 if (readlen < length) {
	    perror("frame file read failure");
	    cerr << "File length: " << length 
		 << " bytes read: " << readlen << endl;
	    part.return_buffer();
         }
         else {
	    part.SetID(gps);
	    part.release(length);
         }
      }
   }
}

